﻿using System;

using Factory2.Units;
using Factory2.Buildings;
using Factory2.Buildings.GDI;
using Factory2.Buildings.NOD;
using Factory2.Buildings.Upgrades;

namespace Factory2
{
    class Program
    {
        static void Main(string[] args)
        {
            #region Observer/Event Pattern Stuff
            /*
            GameUnit exampleUnit = new Soldier();

            BattleMapDisplay battleMap = new BattleMapDisplay(exampleUnit);
            OverviewDisplay overviewMap = new OverviewDisplay(exampleUnit);

            exampleUnit.Move(new Point(10, 10));
            exampleUnit.Move(new Point(20, 20));
            */
            #endregion

            #region Decorator Pattern Stuff
            IBuilding gdiBarracks = new Barracks();
            Console.WriteLine(string.Format("{0} needs {1} power.",
                                            gdiBarracks.Description,
                                            gdiBarracks.Power));

            IBuilding buildingWithOneTurret = new BuildingTurret(new Barracks());
            Console.WriteLine(string.Format("{0} needs {1} power.", 
                                            buildingWithOneTurret.Description, 
                                            buildingWithOneTurret.Power));

            IBuilding aRealFortress = new BuildingTurret(
                                        new BuildingTurret(
                                            new BuildingSAM(
                                                new BuildingLaser(
                                                    new Barracks()
                                                    )
                                                    )
                                                    )
                                                    );

            Console.WriteLine(string.Format("{0} needs {1} power.", 
                                            aRealFortress.Description, 
                                            aRealFortress.Power));

            Console.WriteLine();
            #endregion

            #region Factory Method Stuff
            InfantryFactory barracks = new Barracks();
            InfantryFactory handOfNod = new HandOfNod();

            GameUnit eliteUnitGDI = barracks.BuildUnit(UnitType.EliteUnit);
            Console.WriteLine("Built a {0}, working for {1}, set to {2}.", 
                eliteUnitGDI.Description, 
                eliteUnitGDI.Side, 
                eliteUnitGDI.Behaviour);

            GameUnit eliteUnitNOD = handOfNod.BuildUnit(UnitType.EliteUnit);
            Console.WriteLine("Built a {0}, working for {1}, set to {2}.", 
                eliteUnitNOD.Description, 
                eliteUnitNOD.Side, 
                eliteUnitNOD.Behaviour);
            #endregion
        }
    }
}